from datetime import datetime

from django.http import HttpResponse
from django.utils import timezone
from django.contrib.auth.mixins import LoginRequiredMixin
from django.views.generic import DetailView, ListView
from django.db.models import Q
from rest_framework.decorators import api_view

from calendrier.models import Evenement, EvenementCategorie
from structure.models import Place
from rest_framework.response import Response
from django.db import connection
from itertools import chain
import array as arr
from rest_framework.authtoken.models import Token as Rest_Token


@api_view()
def hello_world(request):
    # token = Rest_Token.objects.get(key=response.data['token'])
    query = Evenement.objects.order_by('-id').select_related('artiste')
    events = Evenement.objects.filter(date__gte=datetime.today()).order_by('date', 'date_de_fin_de_vente',
                                                                           'date_de_fin_de_vente', 'heure_début',
                                                                           'heure_fin', 'titre',
                                                                           'category')
    ev = []
    for event in events:
        print(event.artiste)
        # ev = str(event.artiste)+',' + str(ev)
        ev.append(str(event.artiste))
        # contacts = {
        #     'query': str(event.artiste)
        # }
        print(ev)
        # return Response(event.artiste)
        # evt = events.filter(
        #     Q(tags__nom__iexact=query) | Q(titre__icontains=query) | Q(artiste__nom__icontains=query) | Q(
        #         artiste__prénom__icontains=query)).values()
    contacts = {
        'categories_list': ['ALL'] + list(EvenementCategorie.objects.order_by('-id').values_list('nom', flat=True)),
        'category_name': 'ALL',
        'start_date': datetime.today(),
        'events': events.values(),
        'artiste': ev
    }
    # query = request.GET.get('query', contacts['query'])
    return Response(contacts)


class EventListView(ListView):
    model = Evenement
    paginate_by = 100  # if pagination is desired
    template_name = 'vendeur/2.events.html'

    def get_defaults(self):
        return {
            'categories_list': ['ALL'] + list(EvenementCategorie.objects.order_by('-id').values_list('nom', flat=True)),
            'category_name': 'ALL',
            'start_date': datetime.today(),
            'query': ''
        }

    def get_queryset(self):
        category_name = self.request.GET.get('category_name', self.get_defaults()['category_name'])
        start_date = self.request.GET.get('start_date', self.get_defaults()['start_date'])
        query = self.request.GET.get('query', self.get_defaults()['query'])

        events = Evenement.objects.filter(date__gte=start_date).order_by('date', 'date_de_fin_de_vente',
                                                                         'date_de_fin_de_vente', 'heure_début',
                                                                         'heure_fin', 'titre',
                                                                         'category')
        if category_name != 'ALL':
            events = events.filter(category__nom__iexact=category_name)

        if query:
            events = events.filter(
                Q(tags__nom__iexact=query) | Q(titre__icontains=query) | Q(artiste__nom__icontains=query) | Q(
                    artiste__prénom__icontains=query))
            # print("events ",events)
        return events

    def get_context_data(self, **kwargs):
        context = super(EventListView, self).get_context_data(**kwargs)
        context['categories_list'] = self.request.GET.get('categories_list', self.get_defaults()['categories_list'])
        context['category_name'] = self.request.GET.get('category_name', self.get_defaults()['category_name'])
        context['start_date'] = self.request.GET.get('start_date', self.get_defaults()['start_date'])
        context['query'] = self.request.GET.get('query', self.get_defaults()['query'])
        return context


class EventDetailView(DetailView):
    model = Evenement
    template_name = 'vendeur/3.event.html'

    def _get_all_places(self):
        l = {}
        # response = HttpResponse("Cookie cleared")
        # response.delete_cookie("commande")
        for place in Place.objects.raw("""
           select * from (select structure_place.id,zone_id,type_id, ligne, chaise
           from structure_place left join structure_zone on structure_place.zone_id = structure_zone.id) as all_places 
           left join (select ticketing_reservation.place_id, orders.canceled_at, orders.confirmed_at, orders.paid_at from (select * from ticketing_commande
          where ticketing_commande.evenement_id = {event_id}) as  orders join ticketing_reservation 
           on orders.id = ticketing_reservation.commande_id group by ticketing_reservation.place_id
           HAVING not orders.paid_at isnull) 
           reserved_places on all_places.id = reserved_places.place_id 
           where reserved_places.place_id ISNULL or not reserved_places.canceled_at ISNULL
         or (not reserved_places.confirmed_at ISNULL and  reserved_places.confirmed_at < '{now_before_15_mins}' and reserved_places.paid_at ISNULL);
           """.format(event_id=self.kwargs.get('pk'),
                      now_before_15_mins=timezone.now().replace(tzinfo=None) - timezone.timedelta(minutes=15))):
            l[place.id] = {
                'type': place.type_id,
                'zone': place.zone_id,
                'ligne': place.ligne,
                'chaise': place.chaise
            }

        return l

    def get_context_data(self, **kwargs):
        context = super(EventDetailView, self).get_context_data(**kwargs)
        context['places_meta'] = self._get_all_places()

        return context

    def get(self, request, *args, **kwargs):
        response = super().get(request, *args, **kwargs)
        response.delete_cookie("commande")
        response.delete_cookie("price")
        return response

    def get_queryset(self):
        qs = super(EventDetailView, self).get_queryset()
        return qs.exclude(date_de_début_de_vente__gt=timezone.localtime(timezone.now())).exclude(
            date_de_fin_de_vente=timezone.localtime(timezone.now()))
