from colorfield.fields import ColorField
from django.db import models


class Parameter(models.Model):
    name = models.CharField(max_length=40, unique=True)
    value = models.CharField(max_length=100)
    type = models.CharField(max_length=10, choices=[('int', 'Integer'), ('bool', 'Boolean')])
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    def __str__(self):
        return "{} ({})".format(self.id, self.name)

    class Meta:
        verbose_name = "paramètre"


class PlaceType(models.Model):
    nom = models.CharField(max_length=100, unique=True)
    colour = ColorField(default='#DDDDDD')

    def __str__(self):
        return "{}".format(self.nom)

    class Meta:
        verbose_name = "type de place"
        verbose_name_plural = "types de places"


class MetaInfo(models.Model):
    name = models.CharField(max_length=100)
    value = models.TextField()

    help_text = "This is to store different descriptive information about the OPERA"

    def __str__(self):
        return "{}. {}".format(self.id, self.name)

    class Meta:
        verbose_name = "méta-information"
        verbose_name_plural = "méta-informations"


class EvenementCategorie(models.Model):
    nom = models.CharField(max_length=100, unique=True)

    def __str__(self):
        return "{}. {}".format(self.id, self.nom)

    class Meta:
        verbose_name = "catégorie d\'événements"
        verbose_name_plural = "catégories d\'événements"


class Tag(models.Model):
    nom = models.CharField(max_length=50, unique=True)
    description = models.TextField(blank=True)

    def __str__(self):
        return "{}. {}".format(self.id, self.nom)

    class Meta:
        verbose_name = "mot clé"
        verbose_name_plural = "mots clés"


class PartenaireType(models.Model):
    nom = models.CharField(max_length=100, unique=True)

    def __str__(self):
        return "{}. {}".format(self.id, self.nom)

    class Meta:
        verbose_name = "type de partenaires"
        verbose_name_plural = "types de partenaires"


class CancelReason(models.Model):
    nom = models.CharField(max_length=100, unique=True)

    def __str__(self):
        return "{}. {}".format(self.id, self.nom)

    class Meta:
        verbose_name = "raison d\'annulation"
        verbose_name_plural = "raisons d\'annulation"
