from django.contrib import admin
from django.contrib.admin import SimpleListFilter
from import_export import resources

from calendrier.models import Evenement
from .models import Commande, Reservation


class CommandeResource(resources.ModelResource):
    class Meta:
        model = Commande


class ReadOnlyAdmin(admin.ModelAdmin):
    resource_class = CommandeResource
    readonly_fields = []

    # change_list_template = 'admin/teckiting/change_list.html'

    # def get_export_formats(self):
    #     formats = (
    #         base_formats.CSV,
    #         base_formats.XLS,
    #     )
    #     return [f for f in formats if f().can_export()]
    #
    def change_view(self, request, object_id, extra_context=None):
        if not request.user.is_superuser:
            extra_context = extra_context or {}
            extra_context['show_save_and_continue'] = False
            extra_context['show_save'] = False
        return super().change_view(request, object_id, extra_context=extra_context)

    def get_actions(self, request):
        actions = super().get_actions(request)
        if not request.user.is_superuser and not request.user.groups.filter(name__icontains='comptable').exists():
            if 'delete_selected' in actions:
                del actions['delete_selected']
        return actions

    def has_add_permission(self, request):

        if not request.user.is_superuser:
            return False
        else:
            if '/change/' in str(request):
                return False
            return True

    def has_delete_permission(self, request, obj=None):
        if not request.user.is_superuser:
            return False
        else:
            return True

class ReservationInline(admin.TabularInline):
    model = Reservation
    extra = 1


@admin.register(Commande)
class CommandeAdmin(ReadOnlyAdmin):
    list_display = ['__str__', 'evenement', 'is_invitation', 'get_number', 'paid_at','is_promo', 'flag_reason', 'canceled_at']
    list_filter = ['vendeur', 'evenement', 'is_invitation', 'is_flagged', 'flag_reason', 'created_at']
    readonly_fields = ['places', 'evenement']
    search_fields = ['evenement__titre', 'evenement__artiste__nom', 'evenement__artiste__prénom']
    date_hierarchy = 'created_at'
    inlines = [ReservationInline]

    def get_number(self, obj):
        resv = Reservation.objects.filter(commande=obj.id).count()
       # print('NUMBER: ', resv)
        return resv

    def is_promo(self, obj):
            resv = Reservation.objects.filter(commande=obj.id).first()
            if resv and resv.promo_code:
              return resv.promo_code
            else:
                '-'

    def changelist_view(self, request, extra_context=None):
        extra_context = extra_context or {}
        extra_context['events'] = list(
            Evenement.objects.order_by('-date', 'titre')[:200]
        )
        return super(CommandeAdmin, self).changelist_view(request, extra_context=extra_context)


class PaymentFilter(SimpleListFilter):
    title = 'Paid'
    parameter_name = 'paid'

    def lookups(self, request, model_admin):
        return [('Oui', 'Oui'), ('Non', 'Non')]

    def queryset(self, request, queryset):
        if self.value() == 'Oui':
            return queryset.filter(commande__paid_at__isnull=False)
        else:
            return queryset.filter(commande__paid_at__isnull=True)


class EventFilter(SimpleListFilter):
    title = 'evenement'
    parameter_name = 'evenement'

    def lookups(self, request, model_admin):
        events = Evenement.objects.all()
        return [(c.id, c.titre) for c in events]

    def queryset(self, request, queryset):
        if self.value():
         return queryset.filter(commande__evenement__id=self.value())
        else:
            pass

#@admin.register(Reservation)
class ReservationAdmin(admin.ModelAdmin):
    list_display = ['get_event', 'place', 'commande', 'get_number', 'get_invit',  'promo_code', 'get_paid_at']
    list_filter = [EventFilter, 'place', 'commande', 'price', 'promo_code',  PaymentFilter]

    def get_invit(self, obj):
        return obj.commande.is_invitation

    def get_number(self, obj):
        resv = Reservation.objects.filter(commande=obj.commande.id).count()
       # print('NUMBER: ', resv)
        return resv

    def get_event(self, obj):
        if obj.commande.evenement:
            return obj.commande.evenement.titre
        return '-'

    def get_paid_at(self, obj):
        if obj.commande.paid_at:
            return obj.commande.paid_at
        return '-'

    get_invit.short_description = 'Invitation'
    get_invit.admin_order_field = 'commande__is_invitation'

    get_number.short_description = 'NUMBER'

    def get_cancled(self, obj):
        return obj.commande.paid_at

    get_cancled.short_description = 'Paid'
    get_cancled.admin_order_field = 'commande__paid_at'

    get_event.short_description = 'EVENT'
    get_event.admin_order_field = 'commande__evenement__titre'

    get_paid_at.short_description = 'PAID AT'
    get_paid_at.admin_order_field = 'commande__paid_at'

    pass
