import datetime
import json
import operator
import webbrowser
from functools import reduce

import requests
from django.db import connection

from django import forms
from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.auth.mixins import LoginRequiredMixin
from django.db.models import Q
from django.shortcuts import redirect, render
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils import timezone, formats
from django.utils.html import strip_tags
from django.views.generic import CreateView, ListView
from django.views.generic import DetailView
from django.views.generic import UpdateView, View
import io
from django.http import HttpResponse, HttpRequest, HttpResponseRedirect
import xlsxwriter
from num2words import num2words

from wkhtmltopdf.views import PDFTemplateView

from acteurs.models import Vendeur
from calendrier.models import Evenement, EventPromotionCode
from meta.models import CancelReason
from structure.models import Place
from ticketing.helpers import render_to_pdf
from ticketing.models import Commande, Reservation
from slickpay import InvoiceTransferMerchant
from django.urls import resolve
from datetime import datetime
from django.http import JsonResponse
from django.core.mail import send_mail


class InvitationsForm(forms.Form):
    zones = forms.CharField(
        max_length=1024,
        widget=forms.HiddenInput())


@staff_member_required
def invitations_create_view(request, *args, **kwargs):
    if request.method == 'POST':
        places_data = request.POST.getlist('zones')
        event = Evenement.objects.get(id=kwargs.get('event_id'))
        # if event.active:
        #     return redirect('/admin/calendrier/evenement/')
        zones = []
        count_places = 0
        for z in places_data:
            zones.append(json.loads(z))
        commande = Commande.objects.create(
            evenement=event,
            confirmed_at=timezone.localtime(timezone.now()),
            paid_at=timezone.localtime(timezone.now()),
            is_invitation=True
        )
        rules = []
        objects = []
        print(places_data)
        for zone in zones:
            for line in zone['lines']:
                if line == 'all':
                    rules.append(Q(type__nom__iexact=zone['name']))
                else:
                    rules.append(Q(ligne__iexact=line) & Q(type__nom__iexact=zone['name']))
        if rules:
            places = Place.objects.filter(reduce(operator.or_, rules))
            print(places.__len__())
            for place in places:
                rsv = Reservation(
                    place=place,
                    commande=commande,
                )
                # rsv.create_hash()
                rsv.save()
                count_places = count_places + 1
                # Reservation.objects.bulk_create(objects)
        if count_places > 0:
            event.active = True
            event.save()
        if zones.__len__() > 0:
            return redirect('/admin/calendrier/evenement/')

        return redirect('/admin/calendrier/evenement/')
    else:
        form = InvitationsForm()
    return render(request, 'admin/invitations.html', {'form': form})


def send_email(request):
    # Get the email data from the request, such as the recipient, subject, and message
    recipient = request.POST.get('email')
    subject = 'test email for opera'
    message = 'test email for opera'
    html_message = render_to_string('vendeur/emailTemplate.html', {'OrderNumber': request.session.get('OrderNumber'),
                                                                   'actionCodeDescription': request.session.get(
                                                                       'actionCodeDescription'),
                                                                   'orderId': request.session.get('orderId'),
                                                                   'approvalCode': request.session.get('approvalCode'),
                                                                   'date': request.session.get('date'),
                                                                   'amount': request.session.get('amount'),
                                                                   'status': request.session.get('status'),
                                                                   'completed': request.session.get('completed'),
                                                                   'rejection_reason': request.session.get(
                                                                       'rejection_reason')})
    plain_message = strip_tags(html_message)

    # Send the email using the Django send_mail function
    send = send_mail(subject, plain_message, 'no-reply@operaalger.com.dz', [recipient], html_message=html_message)

    if (send):
        if request.session.get('completed') == 1:
            return redirect(f'https://operaalger.com.dz:8080/events/{str(request.session.get("event_id"))}/commande/{str(request.session.get("pk"))}?invoice_id={str(request.session.get("id_transfer"))}')
        else:
            return redirect(f'https://operaalger.com.dz:8080/events/{str(request.session.get("event_id"))}/commande/{str(request.session.get("pk"))}?invoice_id={str(request.session.get("id_transfer"))}&error={str(request.session.get("error"))}&message={str(request.session.get("message"))}')

        # return redirect(request.session.get('full_url'))
        # return redirect(request.META.get('HTTP_REFERER', '/'))
    # Return a JSON response indicating that the email was sent successfully
    else:
        return redirect('https://google.com')


class CommandListView(ListView):
    model = Commande
    object_list = "commandes_list"
    paginate_by = 100  # if pagination is desired
    template_name = 'vendeur/commandes.html'
    queryset = Commande.objects.filter(is_invitation=False, evenement__isnull=False).order_by('-created_at')

    def get_context_data(self, **kwargs):
        context = super(CommandListView, self).get_context_data(**kwargs)
        context['cancel_reasons'] = CancelReason.objects.all()
        return context


class CommandCreateView(CreateView):
    model = Commande
    template_name = 'vendeur/4.commande.html'
    fields = ['places']

    def _get_all_places(self):
        l = {}
        for place in Place.objects.raw("""
           select * from (select structure_place.id,zone_id,type_id, ligne, chaise
           from structure_place left join structure_zone on structure_place.zone_id = structure_zone.id) as all_places 
           left join (select ticketing_reservation.place_id, orders.canceled_at, orders.confirmed_at, orders.paid_at from (select * from ticketing_commande
          where ticketing_commande.evenement_id = {event_id}) as  orders join ticketing_reservation 
           on orders.id = ticketing_reservation.commande_id group by ticketing_reservation.place_id, orders.paid_at HAVING not orders.paid_at isnull) reserved_places on all_places.id = reserved_places.place_id 
           where reserved_places.place_id ISNULL or not reserved_places.canceled_at ISNULL
           or (not reserved_places.confirmed_at ISNULL and  reserved_places.confirmed_at < '{now_before_15_mins}' and reserved_places.paid_at ISNULL);
           """.format(event_id=self.kwargs.get('event_id'),
                      now_before_15_mins=timezone.now().replace(tzinfo=None) - timezone.timedelta(minutes=15))):
            l[place.id] = {
                'type': place.type_id,
                'zone': place.zone_id,
                'ligne': place.ligne,
                'chaise': place.chaise
            }
        return l

    def get_context_data(self, **kwargs):
        context = super(CommandCreateView, self).get_context_data(**kwargs)

        context['event_id'] = self.kwargs.get('event_id')
        context['evenement'] = Evenement.objects.get(id=self.kwargs.get('event_id'))
        context['places_meta'] = self._get_all_places()

        return context

    def get_initial(self):
        return {'evenement': self.kwargs.get('event_id')}

    def in_dictlist(self, key, value, list):
        for this in list:
            if this[key] == str(value):
                return this
        return {}

    def check_places(self, places):
        all_places = self._get_all_places()
        existed = False
        for place in places:
            if place.id not in all_places:
                existed = True
                break
        return existed

    def form_valid(self, form):
        # recaptcha_response = self.request.POST.get('g-recaptcha-response')
        # data = {
        #     'secret': '6LfH5fwkAAAAAGRU0OWCDoVBZc9jSADf6cDDLfAF',
        #     'response': recaptcha_response
        # }
        # r = requests.post('https://www.google.com/recaptcha/api/siteverify', data=data)
        # result = r.json()
        # print(result)
        total_paid = 0
        i = 0
        promo_codes = form.data['promo_codes']
        places = form.cleaned_data['places']
        event = Evenement.objects.get(id=self.kwargs.get('event_id'))
        vendeur = Vendeur.objects.filter(user__username=self.request.user).first()
        if self.check_places(places):
            form.add_error(None, "Vous avez déjà une commande en cours")
            return self.form_invalid(form)
        commande = Commande.objects.create(
            evenement=event,
            vendeur=vendeur,
            confirmed_at=timezone.localtime(timezone.now()),
            paimenType='CIB/Edahabia'
        )
        print('commande')
        print(commande.paimenType)
        items = []
        special_prices = event.get_special_prices()
        place_price = event.prix_par_default
        for place in places:
            for price, blocks in special_prices.items():
                if place.type.nom in blocks:
                    place_price = price

            reservation = Reservation(
                place=place,
                commande=commande,
                price=place_price,
            )

            if promo_codes:
                promo_applied = self.in_dictlist("id", place.id, json.loads(promo_codes))
                if bool(promo_applied):
                    promo_id = EventPromotionCode.objects.get(code=promo_applied["code_promo"]).id
                    reservation.promo_code_id = promo_id
                    discount_percentage = EventPromotionCode.objects.get(
                        code=promo_applied["code_promo"]).discount_percentage
                    print('discount_percentage')
                    print(discount_percentage)
                    prix1 = float((float(float(reservation.price) * int(discount_percentage))) / 100)
                    print('prix1')
                    print(prix1)
            else:
                prix1 = reservation.price
            reservation.save()

            print('before item')
            item = {
                "name": place.type.nom,  # Use the place name as the item name
                "price": prix1,  # Use the updated place_price as the item price
                "quantity": 1  # Use a default quantity of 1 for now
            }
            print(item)
            items.append(item)

            total_paid = float(float(total_paid) + float(prix1))
            i += 1

        invoice = InvoiceTransferMerchant()
        # return redirect('paiment-create', event_id=self.kwargs.get('event_id'), pk=commande.id)
        print('before data')
        location = reverse("paiment-create", kwargs={'event_id': self.kwargs.get('event_id'), 'pk': commande.id})

        full_url = self.request.build_absolute_uri(location)
        self.request.session['full_url'] = full_url

        print(items)
        data = {
            'amount': total_paid,
            'name': "Client",
            'phone': "000000000",
            'address': "Algeria",
            'url': full_url,
            'items': items
        }

        dt = json.dumps(data, indent=2)
        print('after data ' + dt)
        # print('commande.total_paid '+commande.total_paid)
        # print('json.dumps(data, indent=2) '+json.dumps(data, indent=2))
        res = invoice.createInvoice(data)
        print('json.dumps(res, indent=2) ' + json.dumps(res, indent=2))
        # res['success'] = 0
        if res.get('success') is not None and res['success'] == 1:
            # if res['message'] == "Invoice created successfully.":
            return redirect(res["url"])
        else:
            return redirect('https://operaalger.com.dz/')
        return redirect('https://operaalger.com.dz/')


class PaimentCreateView(UpdateView):
    model = Commande
    fields = ['total_paid', 'paid_at']
    template_name = "vendeur/5.paiment.html"
    invoice = InvoiceTransferMerchant()

    def get_context_data(self, **kwargs):
        context = super(PaimentCreateView, self).get_context_data(**kwargs)
        context["total"] = self.object.get_total()
        context['evenement'] = Evenement.objects.get(id=self.kwargs.get('event_id'))
        id_transfer = self.request.GET.get("invoice_id")
        error = self.request.GET.get("error")
        message = self.request.GET.get("message")
        self.request.session['id_transfer'] = id_transfer
        self.request.session['event_id'] = self.kwargs.get('event_id')
        self.request.session['pk'] = self.kwargs.get('pk')
        self.request.session['error'] = error
        self.request.session['message'] = message

        invoice = InvoiceTransferMerchant()

        res = invoice.invoiceDetail(int(id_transfer))
        print(res)
        self.request.session['success'] = res['success']
        self.request.session['completed'] = res['completed']

        if res['success'] == 1 & res['completed'] == 1:
            print('in get_context_data')
            respData = res['data']['transaction']
            self.request.session['amount'] = respData['amount']
            self.request.session['status'] = respData['status']
            print(respData['log'])
            decoded_response = respData['log']
            self.request.session['expiration'] = decoded_response['expiration']
            self.request.session['cardholderName'] = decoded_response['cardholderName']
            # print(decoded_response)
            # params = decoded_response['params']
            # print(params)

            self.request.session['actionCodeDescription'] = decoded_response['actionCodeDescription']
            self.request.session['OrderNumber'] = decoded_response['OrderNumber']
            self.request.session['date'] = decoded_response['date']
            # assuming self.request.session['date'] contains the datetime string "2023-03-14T07:20:09.281610Z"
            datetime_str = self.request.session['date']

            # convert datetime string to datetime object
            datetime_obj = datetime.strptime(datetime_str, '%Y-%m-%dT%H:%M:%S.%fZ')

            # convert datetime object to human-readable format
            human_date = datetime_obj.strftime('%B %d, %Y %I:%M %p')

            # store the human-readable date in the session
            self.request.session['date'] = human_date
            self.request.session['orderId'] = decoded_response['orderId']
            self.request.session['approvalCode'] = decoded_response['approvalCode']
            print('in get_context_data')
        else:
            print(res['data'])
            self.request.session['rejection_reason'] = res['data']['rejection_reason']
            self.request.session['actionCodeDescription'] = res['data']['transaction']['log']['actionCodeDescription']

        return context

    def get_initial(self):
        return {'total_paid': self.object.get_total()}

    def form_valid(self, form):
        if 'confirm' in self.request.POST:
            order = Commande.objects.get(pk=self.kwargs.get('pk'))
            order.total_paid = form.cleaned_data['total_paid']
            order.paid_at = timezone.localtime(timezone.now())
            order.save(update_fields=['total_paid', 'paid_at'])
            id_transfer = self.request.GET.get("invoice_id")
            error = self.request.GET.get("error")
            message = self.request.GET.get("message")
            self.request.session['id_transfer'] = id_transfer
            self.request.session['event_id'] = self.kwargs.get('event_id')
            self.request.session['pk'] = self.kwargs.get('pk')
            self.request.session['error'] = error
            self.request.session['message'] = message


            invoice = InvoiceTransferMerchant()

            res = invoice.invoiceDetail(int(id_transfer))
            # print('id_transfer ' + id_transfer)
            dt = json.dumps(res, indent=2)
            print('json.dump(res, indent = 6) ' + dt)
            if res['success'] == 0:
                return redirect('confirmationFailed', event_id=self.kwargs.get('event_id'), pk=self.kwargs.get('pk'))

                # return json.dump(res, indent=2)
            else:
                if res['success'] == 1 & res['completed'] == 1:

                    self.request.session['rejection_reason'] = res['data']['rejection_reason']
                    respData = res['data']['transaction']
                    self.request.session['amount'] = respData['amount']
                    self.request.session['status'] = respData['status']
                    decoded_response = respData['log']

                    self.request.session['expiration'] = decoded_response['expiration']
                    self.request.session['cardholderName'] = decoded_response['cardholderName']
                    self.request.session['ErrorMessage'] = decoded_response['ErrorMessage']
                    self.request.session['actionCodeDescription'] = decoded_response['actionCodeDescription']
                    # print(decoded_response)
                    # params = decoded_response['params']
                    # print(params)

                    self.request.session['actionCodeDescription'] = decoded_response['actionCodeDescription']
                    self.request.session['OrderNumber'] = decoded_response['OrderNumber']
                    self.request.session['date'] = decoded_response['date']
                    # assuming self.request.session['date'] contains the datetime string "2023-03-14T07:20:09.281610Z"
                    datetime_str = self.request.session['date']

                    # convert datetime string to datetime object
                    datetime_obj = datetime.strptime(datetime_str, '%Y-%m-%dT%H:%M:%S.%fZ')

                    # convert datetime object to human-readable format
                    human_date = datetime_obj.strftime('%B %d, %Y %I:%M %p')

                    # store the human-readable date in the session
                    self.request.session['date'] = human_date
                    self.request.session['orderId'] = decoded_response['orderId']
                    self.request.session['approvalCode'] = decoded_response['approvalCode']
                    Commande.objects.filter(pk=self.kwargs.get('pk')).update(
                        log=decoded_response)
                    print("Commande.objects.filter(pk=self.kwargs.get('pk')).log")
                    # print(Commande.objects.filter(pk=self.kwargs.get('pk')).log)

                    return redirect('confirmation', event_id=self.kwargs.get('event_id'), pk=self.kwargs.get('pk'))

            # return redirect('https://operaalger.com.dz/')
        elif 'cancel' in self.request.POST:
            Commande.objects.filter(pk=self.kwargs.get('pk')).update(
                canceled_at=timezone.localtime(timezone.now())
            )
            return redirect('https://operaalger.com.dz/')
        else:
            return redirect('https://operaalger.com.dz/')


class CommandDetailView(DetailView):
    model = Commande
    template_name = 'vendeur/6.confirmation.html'

    def get_context_data(self, **kwargs):
        context = super(CommandDetailView, self).get_context_data(**kwargs)
        context['cancel_reasons'] = CancelReason.objects.all()
        return context


class CommandDetailFailView(DetailView):
    model = Commande
    template_name = 'vendeur/7.failConfirmation.html'


class TicketDetailView(DetailView):
    model = Reservation
    template_name = 'vendeur/ticket.html'


def pdfview(request, pk):
    return render_to_pdf(
        'vendeur/ticket.html',
        {
            'pagesize': 'A4',
            'mylist': "test",
        }
    )


def flagerCommande(request, *args, **kwargs):
    if request.method == 'GET':
        commande = Commande.objects.get(id=kwargs.get('pk'))
        cancel_id = kwargs.get('cancel_id')
        if commande.is_flagged:
            return redirect('/')
        else:
            commande.is_flagged = True
            if cancel_id:
                cancel_reason = CancelReason.objects.get(id=kwargs.get('cancel_id'))
                commande.flag_reason = cancel_reason
            commande.save()
        return redirect('/commandes/')
    else:
        return redirect('/commandes/')


class TicketPdfView(DetailView):
    model = Commande
    template_name = 'admin/invitations_list.html'


class TicketPdfViewPdf(PDFTemplateView):
    model = Commande
    template_name = 'admin/tickets_pdf.html'
    cmd_options = {
        'page-size': 'A4',
        'margin-top': 0,
        'margin-left': 0,
        'margin-bottom': 0,
        'margin-right': 0,
    }

    # show_content_in_browser = True

    def get_context_data(self, **kwargs):
        context = super(TicketPdfViewPdf, self).get_context_data(**kwargs)
        context['object'] = Commande.objects.get(id=self.kwargs.get('pk'))
        return context

    def get_filename(self):
        return '{0}-{1}.pdf'.format('invitations',
                                    timezone.localtime(timezone.now()).replace(microsecond=0, tzinfo=None), )


class ReportsView(ListView):
    model = Evenement
    template_name = 'admin/reports.html'


class ReportsEventView(View):
    def get(self, request, **kwargs):
        output = io.BytesIO()
        workbook = xlsxwriter.Workbook(output)
        worksheet = workbook.add_worksheet()
        event_id = self.kwargs['event_id']
        prices_classes = Evenement.objects.get(pk=event_id)
        worksheet.set_column('B:G', 20)
        worksheet.insert_textbox('B2', prices_classes.titre.title() + ', le ' + str(
            formats.date_format(prices_classes.date, format="DATE_FORMAT", use_l10n=True)).title(),
                                 {'x_scale': 1.5, 'y_scale': 0.25,
                                  'bold': True, 'font_size': 22,
                                  'align': {'vertical': 'middle',
                                            'horizontal': 'center'
                                            },
                                  'fill': {'color': 'white'},
                                  'line': {'color': 'white'}})
        index = 5
        cell_format = workbook.add_format({'bold': True, 'font_size': 14, 'border': 1, 'text_wrap': 1})
        cell_format.set_align('center')
        cell_format.set_align('vcenter')
        money_format = workbook.add_format({'bold': False, 'font_size': 12, 'border': 1})
        money_format.set_align('center')
        money_format.set_align('vcenter')
        number_format = workbook.add_format({'bold': False, 'font_size': 12, 'border': 1})
        number_format.set_align('center')
        number_format.set_align('vcenter')
        worksheet.set_column('B:D', 20)
        money_format.set_num_format('"DZD" #,##0.00')
        worksheet.write_string('B' + str(index), 'Prix', cell_format)
        worksheet.write_string('C' + str(index), 'Qté billetterie vendue', cell_format)
        worksheet.write_string('D' + str(index), 'Total', cell_format)
        for price, value in prices_classes.get_special_prices().items():
            count_places = prices_classes.get_available_count_price(price)
            if count_places > 0:
                worksheet.write_number('B' + str(index + 1), price, money_format)
                worksheet.write_number('C' + str(index + 1), count_places, number_format)
                worksheet.write_formula('D' + str(index + 1), '{=B' + str(index + 1) + '*C' + str(index + 1) + '}',
                                        money_format)
                index += 1
        worksheet.write_string('C' + str(index + 2), 'Total', cell_format)
        worksheet.write_formula('D' + str(index + 2), '{=SUM(D' + str(4) + ':D' + str(index) + ')}', money_format)
        worksheet.insert_textbox('B' + str(index + 4),
                                 'Ce document a été généré par Tick-IN, le système de billetterie de l’Opéra d’Alger',
                                 {'x_scale': 5, 'y_scale': 0.25,
                                  'align': {'vertical': 'middle',
                                            'horizontal': 'center'
                                            },
                                  'fill': {'color': 'white'}, 'font': {'bold': True}})
        workbook.close()
        output.seek(0)
        title = prices_classes.titre.translate({ord(c): " " for c in "!@#$%^&*()[]{};:,./<>?\|`~-=_+"})

        filename = 'rapport ' + title + '.xlsx'
        response = HttpResponse(
            output,
            content_type='application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'
        )
        response['Content-Disposition'] = 'attachment; filename=%s' % filename

        return response


def get_all_details(min, max):
    with connection.cursor() as cursor:
        cursor.execute("""select calendrier_evenement.id,
       titre,
       commande_id,
       zone_id,
       ligne,
       chaise,
       nom,
       code,
       paid_at from calendrier_evenement
join
    (SELECT *
FROM   (
                 SELECT    *,
                           structure_place.id AS id
                 FROM      structure_place
                 LEFT JOIN meta_placetype
                 ON        structure_place.type_id = meta_placetype.id) AS all_places
JOIN
       (
                SELECT   reservs.place_id,
                         reservs.price,
                         reservs.commande_id,
                         reservs.code,
                         orders.evenement_id,
                         orders.paid_at
                FROM     (
                                SELECT *
                                FROM   ticketing_commande
                                WHERE NOT ticketing_commande.paid_at isnull  AND (ticketing_commande.paid_at >= %s) AND (ticketing_commande.paid_at < %s)) AS orders
                JOIN
                         (
                                SELECT * FROM (
    SELECT * FROM   ticketing_reservation
                                WHERE  NOT ticketing_reservation.price isnull) plcs
                                 LEFT JOIN calendrier_eventpromotioncode
                                      on plcs.promo_code_id=calendrier_eventpromotioncode.id  ) reservs
                ON       orders.id = reservs.commande_id
                GROUP BY reservs.place_id, orders.paid_at
                HAVING   NOT orders.paid_at isnull) reservations
ON     all_places.id = reservations.place_id ) details on calendrier_evenement.id= details.evenement_id order by titre, paid_at;""",
                       [min, max])
        return cursor.fetchall()


class ReportsMonthlyView(View):
    def get(self, request, **kwargs):
        output = io.BytesIO()
        workbook = xlsxwriter.Workbook(output)
        worksheet = workbook.add_worksheet('État des recettes')
        worksheet.set_column('B:C', 20)
        worksheet.set_column('C:J', 18)
        year = self.kwargs['year']
        month = self.kwargs['month']
        month_events = Evenement.objects.filter(date__month=month, date__year=year)
        worksheet.insert_textbox('B2', 'ÉTAT DES RECETTES DE LA BILLETTERIE VENDUE PÉRIODE  ' + str(
            formats.date_format(datetime.datetime(year=int(year), month=int(month), day=1), format="YEAR_MONTH_FORMAT",
                                use_l10n=True)).capitalize(),
                                 {'x_scale': 5, 'y_scale': 0.25,
                                  'bold': True, 'font_size': 16,
                                  'align': {'vertical': 'middle',
                                            'horizontal': 'center'
                                            },
                                  'fill': {'color': 'white'},
                                  'line': {'color': 'white'}, 'font': {'bold': True}})
        cell_format = workbook.add_format({'bold': True, 'font_size': 12, 'border': 1, 'text_wrap': 1})
        cell_format.set_align('center')
        cell_format.set_align('vcenter')
        cell_format_2 = workbook.add_format({'bold': False, 'font_size': 10, 'border': 1, 'text_wrap': 1})
        cell_format_2.set_align('center')
        cell_format_2.set_align('vcenter')
        money_format = workbook.add_format({'bold': False, 'font_size': 10, 'border': 1})
        money_format.set_align('center')
        money_format.set_align('vcenter')
        number_format = workbook.add_format({'bold': False, 'font_size': 10, 'border': 1})
        number_format.set_align('center')
        number_format.set_align('vcenter')
        money_format.set_num_format('"DZD" #,##0.00')
        row_price = []
        index = 5
        month_int = int(month)
        year_int = int(year)
        min = datetime.datetime(year_int, month_int, 1)
        max = datetime.datetime(year_int + int(month_int / 12), (month_int % 12) + 1, 1)
        worksheet.write_string('B' + str(index), 'Nature des operations', cell_format)
        worksheet.write_string('C' + str(index), 'Nbr tickets vendu', cell_format)
        worksheet.write_string('D' + str(index), 'P.U HT', cell_format)
        worksheet.write_string('E' + str(index), 'TVA 9%', cell_format)
        worksheet.write_string('F' + str(index), 'Montant HT', cell_format)
        worksheet.write_string('G' + str(index), 'Montant TVA 9%', cell_format)
        worksheet.write_string('H' + str(index), 'Montant TTC', cell_format)
        worksheet.write_string('I' + str(index), 'Observation', cell_format)
        for event in month_events:
            for price, value in event.get_special_prices().items():
                puht_price = round(price / 1.09, 2)
                tva_price = price - puht_price
                count_places = event.get_available_count_price(price)
                if count_places > 0:
                    row_price.append((tva_price + puht_price) * count_places)
                    if event.category:
                        title = event.category.nom + ' ' + event.titre + ' \n ' + str(event.date.strftime('%d/%m/%Y'))
                    else:
                        title = event.titre + ' \n ' + str(event.date.strftime('%d/%m/%Y'))
                    #  print('hey ', title, count_places)
                    worksheet.write_string('B' + str(index + 1), title, cell_format_2)
                    worksheet.write_number('C' + str(index + 1), count_places, number_format)
                    worksheet.write_number('D' + str(index + 1), puht_price, money_format)
                    worksheet.write_number('E' + str(index + 1), tva_price, money_format)
                    worksheet.write_formula('F' + str(index + 1), '{=C' + str(index + 1) + '*D' + str(index + 1) + '}',
                                            money_format)
                    worksheet.write_formula('G' + str(index + 1), '{=C' + str(index + 1) + '*E' + str(index + 1) + '}',
                                            money_format)
                    worksheet.write_formula('H' + str(index + 1), '{=F' + str(index + 1) + '+G' + str(index + 1) + '}',
                                            money_format)
                    worksheet.write_blank('I' + str(index + 1), '', cell_format_2)
                    index += 1
            # if merged_cells != index - 1 and index != 6:
            #     worksheet.merge_range('B' + str(merged_cells+1) + ':B' + str(index), title, cell_format_2)

        money_format_2 = workbook.add_format({'bold': True, 'font_size': 10, 'border': 1})
        money_format_2.set_align('center')
        money_format_2.set_align('vcenter')
        money_format_2.set_num_format('"DZD" #,##0.00')
        number_format_2 = workbook.add_format({'bold': True, 'font_size': 10, 'border': 1})
        last_money = workbook.add_format({'bold': True, 'font_size': 10, 'border': 1})
        number_format_2.set_align('center')
        number_format_2.set_align('vcenter')
        worksheet.write_string('B' + str(index + 1), 'TOTAUX', cell_format)
        worksheet.write_formula('C' + str(index + 1), '{=SUM(C6:C' + str(index) + ')}', number_format_2)
        worksheet.write_formula('D' + str(index + 1), '{=SUM(D6:D' + str(index) + ')}', money_format_2)
        worksheet.write_formula('E' + str(index + 1), '{=SUM(E6:E' + str(index) + ')}', money_format_2)
        worksheet.write_formula('F' + str(index + 1), '{=SUM(F6:F' + str(index) + ')}', money_format_2)
        worksheet.write_formula('G' + str(index + 1), '{=SUM(G6:G' + str(index) + ')}', money_format_2)
        worksheet.write_formula('H' + str(index + 1), '{=SUM(H6:H' + str(index) + ')}', money_format_2)
        worksheet.write_blank('I' + str(index + 1), '', cell_format_2)
        somme = sum(row_price[0:len(row_price)])
        worksheet.insert_textbox('B' + str(index + 4),
                                 'Arrêté le présent état à la somme de : ' + num2words(somme, lang='fr_DZ',
                                                                                       to='currency').replace("dinards",
                                                                                                              "Dinars").title(),
                                 {'x_scale': 5, 'y_scale': 0.25,
                                  'align': {'vertical': 'middle',
                                            'horizontal': 'center'
                                            },
                                  'fill': {'color': 'white'}, 'font': {'bold': True}})
        worksheet.insert_textbox('B' + str(index + 6),
                                 'Ce document a été généré par Tick-IN, le système de billetterie de l’Opéra d’Alger',
                                 {'x_scale': 5, 'y_scale': 0.25,
                                  'align': {'vertical': 'middle',
                                            'horizontal': 'center'
                                            },
                                  'fill': {'color': 'white'}, 'font': {'bold': True}})
        worksheet2 = workbook.add_worksheet('Rapport détaillé')
        index = 1

        cell_format.set_bg_color('black')
        cell_format.set_font_color('white')
        string_format = workbook.add_format({'bold': False, 'font_size': 12, 'border': 1})
        string_format.set_align('center')
        string_format.set_align('vcenter')

        details = get_all_details(min, max)
        worksheet2.write_string('B' + str(index), 'Event ID', cell_format)
        worksheet2.write_string('C' + str(index), "Nom de l'événement", cell_format)
        worksheet2.write_string('D' + str(index), 'Commande ID', cell_format)
        worksheet2.write_string('E' + str(index), 'Zone', cell_format)
        worksheet2.write_string('F' + str(index), 'Rangée', cell_format)
        worksheet2.write_string('G' + str(index), 'Place', cell_format)
        worksheet2.write_string('H' + str(index), 'Classe de prix', cell_format)
        worksheet2.write_string('I' + str(index), 'Code promo appliqué', cell_format)
        worksheet2.write_string('J' + str(index), 'Payée à', cell_format)
        index = index + 1
        for row in details:
            worksheet2.write_string('B' + str(index), str(row[0]), string_format)
            worksheet2.write_string('C' + str(index), str(row[1]), string_format)
            worksheet2.write_string('D' + str(index), str(row[2]), string_format)
            worksheet2.write_string('E' + str(index), str(row[3]), string_format)
            worksheet2.write_string('F' + str(index), str(row[4]), string_format)
            worksheet2.write_string('G' + str(index), str(row[5]), string_format)
            worksheet2.write_string('H' + str(index), str(row[6]), string_format)
            if row[7]:
                worksheet2.write_string('I' + str(index), str(row[7]), string_format)
            else:
                worksheet2.write_string('I' + str(index), '-', string_format)
            worksheet2.write_string('J' + str(index), str(row[8].strftime("%Y-%m-%d %H:%M")), string_format)
            index = index + 1
        worksheet2.set_column('B:J', 20)
        worksheet2.set_column('C:C', 30)
        workbook.close()
        output.seek(0)
        filename = 'rapport_monthly_' + month + '_' + year + '.xlsx'
        response = HttpResponse(
            output,
            content_type='application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'
        )
        response['Content-Disposition'] = 'attachment; filename=%s' % filename

        return response
